'
' A program to sort functions within a source program
' It will also sort the function declarations within the PROLOG
' The first declared function (ie, the entry function) is not sorted.
'
PROGRAM "sourcesort"
VERSION "0.0001"
CONSOLE

IMPORT "xst_s"
IMPORT "xsx_s"
'IMPORT "xio_s"

DECLARE FUNCTION Entry ()
DECLARE FUNCTION GrabFunctionName (line$, @fn$)
EXPORT
DECLARE FUNCTION SortSourceFile (fin$, fout$)
DECLARE FUNCTION SourceSort (@in$[], @out$[])
END EXPORT


FUNCTION Entry ()

	IF LIBRARY(0) THEN RETURN
	
	XstGetCommandLineArguments (@argc, @argv$[])
	
	IF argc < 3 THEN
		GOSUB Help
		RETURN
	END IF

	fin$  = argv$[1]	
	IF XstFileExists (fin$) THEN
	  ? "Sourcesort error. File not found."
		RETURN
	END IF
	
	fout$ = argv$[2]
	IF SortSourceFile (fin$, fout$) THEN
	  ? "Sourcesort error."
	ELSE
	  ? "Sourcesort finished."
	END IF

SUB Help
	PRINT "Sort xblite program functions and declarations:"
	PRINT "Use:"
	PRINT	"      sortsource infile outfile"
	PRINT
END SUB

  a$ = INLINE$ ("Press Enter to quit >")

END FUNCTION

FUNCTION GrabFunctionName (line$, @fn$)

' this assumes there is only one space character
' between function name and arg list, eg
' myfunc (arglist)

	IFZ line$ THEN RETURN ($$TRUE)
	fn$ = ""
	buffer$ = LTRIM$ (line$)

	p = INSTR (buffer$, "(")
	IFZ p THEN RETURN ($$TRUE)
	fn$ = TRIM$ (LEFT$ (buffer$, p - 1))
	p = RINSTR (fn$, " ")
	IFZ p THEN RETURN ($$TRUE)
	fn$ = TRIM$ (MID$ (fn$, p))

END FUNCTION

FUNCTION SortSourceFile (fin$, fout$)

	IFZ fin$ THEN RETURN $$TRUE
	IFZ fout$ THEN RETURN $$TRUE
	IF XstLoadStringArray (fin$, @in$[]) THEN RETURN $$TRUE
	
	upp = UBOUND(in$[])
	IFZ in$[] THEN RETURN $$TRUE

	SourceSort (@in$[], @out$[])
	IFZ out$[] THEN RETURN $$TRUE
	
	XstSaveStringArrayCRLF (fout$, @out$[])

END FUNCTION

FUNCTION SourceSort (@in$[], @out$[])

	IFZ in$[] THEN RETURN $$TRUE

	DIM prog[256,]	' program sections
	DIM t$[512]			' temp array for function sections
	DIM fn$[256]		' array for function names
	DIM dn$[256,1]	' array for declaration names and line number
		
	upProg = 256
	upT    = 512
	upFn   = 256
	upDn   = 256

	upp = UBOUND(in$[])
	
	FOR i = 0 TO upp							' process each line
		s$ = in$[i]
		line$ = TRIM$ (s$)
		IFZ line$ THEN							' skip empty lines
			ATTACH s$ TO t$[count]
			INC count 
			IF (count > upT) THEN	
				upT = upT << 1
				REDIM t$[upT]
			END IF	
			DO NEXT
		END IF

		XstTranslateChars (@line$, "\t", " ")
		XstReplace (@line$, "(", " ( ", 0)
		DO
		LOOP UNTIL XstReplace (@line$, "  ", " ", 0) = 0
		
		first$ = XstParse$ (line$, " ", 1)
		
		SELECT CASE first$
		
			CASE "DECLARE", "INTERNAL" :
			
				' grab declaration function names
				IF GrabFunctionName (line$, @dec$) THEN RETURN $$TRUE
				dn$[dnCount,0] = dec$
				dn$[dnCount,1] = STRING$(i)
				INC dnCount
				IF (dnCount > upDn) THEN	
					upDn = upDn << 1
					REDIM dn$[upDn, 1]
				END IF
	
				ATTACH s$ TO t$[count]
				INC count
				IF (count > upT) THEN	
					upT = upT << 1
					REDIM t$[upT]
				END IF	
				
			CASE "FUNCTION", "CFUNCTION" :
				' first function defined, capture PROLOG section
				IFZ func_count THEN			
					REDIM t$[count-1]
					ATTACH t$[] TO prog[func_count,]
					INC func_count
	
					IF (func_count > upProg) THEN	
						upProg = upProg << 1
						REDIM prog[upProg,]
					END IF					
					
					count = 0
					DIM t$[512]
					upT = 512
					ATTACH s$ TO t$[count]
					INC count
					
				ELSE
					ATTACH s$ TO t$[count]
					INC count
					IF (count > upT) THEN	
						upT = upT << 1
						REDIM t$[upT]
					END IF		
				END IF
				
				' now grab function name
				IF GrabFunctionName (line$, @fn$) THEN RETURN $$TRUE
				fn$[fnCount] = fn$
				INC fnCount
				IF (fnCount > upFn) THEN	
					upFn = upFn << 1
					REDIM fn$[upFn]
				END IF		

			CASE "END" :
				sec$ = XstParse$ (line$, " ", 2)
				
				SELECT CASE sec$
					
					CASE "FUNCTION", "PROGRAM" :
						ATTACH s$ TO t$[count]
						REDIM t$[count]
						
						ATTACH t$[] TO prog[func_count,]
						INC func_count
						IF (func_count > upProg) THEN	
							upProg = upProg << 1
							REDIM prog[upProg,]
						END IF		
						
						count = 0
						DIM t$[512]
						upT = 512
						
					CASE ELSE :
						ATTACH s$ TO t$[count]
						INC count
						IF (count > upT) THEN	
							upT = upT << 1
							REDIM t$[upT]
						END IF		
						
				END SELECT
				
			CASE ELSE :
			
				ATTACH s$ TO t$[count]
				INC count 
				IF (count > upT) THEN	
					upT = upT << 1
					REDIM t$[upT]
				END IF		
		END SELECT
	NEXT i
	
	REDIM prog[func_count-1,]
	REDIM fn$[fnCount-1]
	REDIM dn$[dnCount-1, 1]
	
	' did we parse the same number of declared functions and code functions
	IF dnCount + 2 <> func_count THEN RETURN $$TRUE
	
	upp = UBOUND(prog[])						' upper bound of prog[]
	DIM newprog[upp,]								' create output array newprog[]

	SWAP newprog[0,], prog[0,]			' swap prolog
	SWAP newprog[upp,], prog[upp,]	' swap end program
	
	' ok, now sort the declaration functions in PROLOG
	
	uppT = UBOUND(dn$[])
	DIM temp$[uppT]
	FOR i = 0 TO uppT
		temp$[i] = dn$[i,0]
	NEXT i

	flags = $$SortIncreasing | $$SortAlphabetic | $$SortCaseInsensitive
	temp$[0] = "aaa" + temp$[0]						' first declaration is entry function, do not move it
	DIM order[0]
	IF XstQuickSort (@temp$[], @order[], 0, UBOUND(temp$[]), flags) THEN RETURN $$TRUE
	temp$[0] = LCLIP$ (temp$[0], 3)
	entry$ = temp$[0]	

	SWAP newprog[0,], temp$[]							' copy PROLOG to temp$[]
	XstCopyArray (@temp$[], @prolog$[])		' make a second copy of PROLOG
	
	uppOrd = UBOUND(order[])							' sort declaration lines according to order[] array
	FOR i = 0 TO uppOrd
		line1 = XLONG (dn$[order[i],1])
		line2 = XLONG (dn$[i,1])
		SWAP prolog$[line2], temp$[line1]
	NEXT i
	
	SWAP prolog$[], newprog[0,]						' swap result back to newprog[]
	
	' before sorting functions, if required, move entry function to top
	IF fn$[0] <> entry$ THEN
		uppFn = UBOUND(fn$[])
		FOR i = 1 TO uppFn
			IF fn$[i] = entry$ THEN
				SWAP fn$[0], fn$[i]							' swap function names
				SWAP prog[1,], prog[i+1,]				' first function is at prog[1,]
				ok = $$TRUE
				EXIT FOR
			END IF
		NEXT i
		IF !ok THEN RETURN $$TRUE   				' function not found!
	END IF
	
	' sort function names
	flags = $$SortIncreasing | $$SortAlphabetic | $$SortCaseInsensitive
	fn$[0] = "aaa" + fn$[0]
	DIM order[0]
	IF XstQuickSort (@fn$[], @order[], 0, UBOUND(fn$[]), flags) THEN RETURN $$TRUE
	fn$[0] = LCLIP$ (fn$[0], 3)
	
	' now sort source functions by order array
	' PROLOG is prog[0,], so start at prog[i+1,]
	
	uppOrd = UBOUND(order[])
	FOR i = 0 TO uppOrd
		order = order[i]
		SWAP newprog[i+1,], prog[order+1,]
	NEXT i

	' reassemble output array out$[] from newprog[]
	upp = UBOUND(newprog[])
	lines = 0	
	FOR i = 0 TO upp
		IFZ newprog[i,] THEN DO NEXT
		lines = lines + UBOUND(newprog[i,]) + 1
	NEXT i
	
	IFZ lines THEN RETURN $$TRUE
	
	DIM out$[lines]
	
	line = 0
	FOR i = 0 TO upp
		DIM temp$[]
		ATTACH newprog[i,] TO temp$[]
		uppTemp = UBOUND(temp$[])
		FOR j = 0 TO uppTemp
			ATTACH temp$[j] TO out$[line]
			INC line
		NEXT j
	NEXT i

END FUNCTION

END PROGRAM