'
' ####################
' #####  PROLOG  #####
' ####################
'
' A library which can be used to move or resize child controls at runtime. 
' All child controls must have assigned $$WS_CLIPSIBLINGS style.
' First attach the form (window) using AttachForm().
' Then attach a control using AttachControl().
' Do not attach controls until all child controls have been added to the form.
' A $$SIZER_CONTROLMODIFIED  message is sent to the form when a child control
' has been resized or moved. 
' 
'
PROGRAM	"sizer"
VERSION "0.0002"
CONSOLE
'
	IMPORT "xst"				' Standard library : required by most programs
	IMPORT "xsx"				' Extended standard library
'	IMPORT "xio"				' Console input/ouput library

'	IMPORT "xst_s.lib"
' IMPORT "xsx_s.lib"
' IMPORT "xio_s.lib"

  IMPORT "gdi32"			' gdi32.dll
	IMPORT "user32"		  ' user32.dll
	IMPORT "kernel32"	  ' kernel32.dll
'	IMPORT "shell32"		' shell32.dll
'	IMPORT "msvcrt"		  ' msvcrt.dll
	IMPORT  "comctl32"	' comctl32.dll			: common controls library

EXPORT
DECLARE FUNCTION Sizer ()
DECLARE FUNCTION AttachForm (hWnd)
DECLARE FUNCTION AttachControl (hCtrl)
DECLARE FUNCTION SetHandleSize (size)
DECLARE FUNCTION SetGridSize (size)
DECLARE FUNCTION SetHandleOffset (offset)
END EXPORT

DECLARE FUNCTION BeginSize (x, y)
DECLARE FUNCTION DrawSizeRect (bEraseOnly)
DECLARE FUNCTION EndSize ()
DECLARE FUNCTION HideHandles ()
DECLARE FUNCTION ShowHandles ()
DECLARE FUNCTION SizeControl ()
DECLARE FUNCTION BringToFront (hWnd)
DECLARE FUNCTION MoveHandle (hWnd, x, y)

DECLARE FUNCTION DrawLine (hdc, left, top, right, bottom, style, width, color)

DECLARE FUNCTION CreateHandles ()
DECLARE FUNCTION TopLeftHandleProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION LeftHandleProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION TopRightHandleProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION TopHandleProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION RightHandleProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION BottomLeftHandleProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION BottomHandleProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION BottomRightHandleProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION MoverHandleProc (hWnd, msg, wParam, lParam)
DECLARE FUNCTION DestroyHandles ()

EXPORT
$$SIZER_CONTROLMODIFIED = 2000	' sizing operation occurred, control handle returned in lParam
END EXPORT

FUNCTION Sizer ()

	
END FUNCTION


FUNCTION AttachForm (hWnd)

	SHARED handles[]
	STATIC init
	SHARED hCurFrm, handleSize, moverSize, gridSize, handleOffset
	SHARED hInst

	hCurFrm = hWnd

	IFZ init THEN
		gridSize = 2
		handleSize = 8
		moverSize = handleSize * 2
		handleOffset = 3
		hInst = GetModuleHandleA (0)
		init = $$TRUE
	END IF

'	IF handles[] THEN DestroyHandles()
'	IFZ handles[] THEN CreateHandles()

END FUNCTION

FUNCTION AttachControl (hCtrl)

	SHARED hCurCtrl, hCurFrm
	SHARED handles[]
	SHARED handleSize, moverSize, gridSize, handleOffset
	SHARED hInst
	
	RECT curctl

	'Save the current control
	hCurCtrl = hCtrl
	
	GetClientRect (hCurCtrl, &curctl)
	MapWindowPoints (hCurCtrl, hCurFrm, &curctl, 2) 
	curctlWidth = curctl.right - curctl.left
	curctlHeight = curctl.bottom - curctl.top
	
	IF handles[] THEN DestroyHandles()
	IFZ handles[] THEN CreateHandles()

	'Put all the handles in place around the control
	
	'top left
	MoveHandle (handles[0], curctl.left - handleSize - handleOffset, curctl.top - handleSize - handleOffset)
	'top
	MoveHandle (handles[1], curctl.left + ((curctlWidth - handleSize) \ 2), curctl.top - handleSize - handleOffset)
	'top right
	MoveHandle (handles[2], curctl.left + curctlWidth + handleOffset, curctl.top - handleSize - handleOffset)
	
	'left
	MoveHandle (handles[3], curctl.left - handleSize - handleOffset, curctl.top + ((curctlHeight - handleSize) \ 2))
	'right
	MoveHandle (handles[4], curctl.left + curctlWidth + handleOffset, curctl.top + ((curctlHeight - handleSize) \ 2))
	
	'bottom left
	MoveHandle (handles[5], curctl.left - handleSize - handleOffset, curctl.top + curctlHeight + handleOffset)
	'bottom
	MoveHandle (handles[6], curctl.left + ((curctlWidth - handleSize) \ 2), curctl.top + curctlHeight + handleOffset)
	'bottom right
	MoveHandle (handles[7], curctl.left + curctlWidth + handleOffset, curctl.top + curctlHeight + handleOffset)
	
	'mover
	MoveHandle (handles[8], curctl.left + ((curctlWidth - moverSize) \ 2), curctl.top + ((curctlHeight - moverSize) \ 2))
	
	'Show them!
	ShowHandles ()

END FUNCTION

FUNCTION HideHandles ()

	SHARED handles[]
	
	upp = UBOUND(handles[])
	FOR i = 0 TO upp
		ShowWindow (handles[i], $$SW_HIDE)
	NEXT i
	
END FUNCTION

FUNCTION DrawSizeRect (bEraseOnly)

	SHARED RECT rc
	SHARED RECT sizerc
	SHARED hCurCtrl
	SHARED hCurFrm
	SHARED bErase
	SHARED bSizing
	RECT rect

    'Draw the sizing rectangle during a size or move operation
		
		hDC = GetDC (hCurFrm)
		oldROP = SetROP2 (hDC, $$R2_NOTXORPEN)

    IF bErase = $$TRUE THEN
        IF bSizing = $$FALSE THEN
					DrawLine (hDC, rc.left, rc.top, rc.right, rc.top, $$PS_SOLID, 2, 0)
					DrawLine (hDC, rc.left, rc.top, rc.left, rc.bottom, $$PS_SOLID, 2, 0)
					DrawLine (hDC, rc.left, rc.bottom, rc.right, rc.bottom, $$PS_SOLID, 2, 0)
					DrawLine (hDC, rc.right, rc.top, rc.right, rc.bottom, $$PS_SOLID, 2, 0)
        ELSE
					DrawLine (hDC, sizerc.left, sizerc.top, sizerc.right, sizerc.top, $$PS_SOLID, 2, 0)
					DrawLine (hDC, sizerc.left, sizerc.top, sizerc.left, sizerc.bottom, $$PS_SOLID, 2, 0)
					DrawLine (hDC, sizerc.left, sizerc.bottom, sizerc.right, sizerc.bottom, $$PS_SOLID, 2, 0)
					DrawLine (hDC, sizerc.right, sizerc.top, sizerc.right, sizerc.bottom, $$PS_SOLID, 2, 0)
        END IF
    END IF
		
    IF bEraseOnly = $$TRUE THEN
        bErase = $$FALSE
        GOTO Cleanup
    END IF
		
		GetClientRect (hCurCtrl, &rect)
		
		MapWindowPoints (hCurCtrl, hCurFrm, &rect, 2) 

    rc.left = rect.left
    rc.top = rect.top
		
		width = rect.right - rect.left
		height = rect.bottom - rect.top
    rc.right = rc.left + width
    rc.bottom = rc.top + height
		
    IF bSizing = $$FALSE THEN
			DrawLine (hDC, rc.left, rc.top, rc.right, rc.top, $$PS_SOLID, 2, 0)
			DrawLine (hDC, rc.left, rc.top, rc.left, rc.bottom, $$PS_SOLID, 2, 0)
			DrawLine (hDC, rc.left, rc.bottom, rc.right, rc.bottom, $$PS_SOLID, 2, 0)
			DrawLine (hDC, rc.right, rc.top, rc.right, rc.bottom, $$PS_SOLID, 2, 0)
    ELSE
			DrawLine (hDC, sizerc.left, sizerc.top, sizerc.right, sizerc.top, $$PS_SOLID, 2, 0)
			DrawLine (hDC, sizerc.left, sizerc.top, sizerc.left, sizerc.bottom, $$PS_SOLID, 2, 0)
			DrawLine (hDC, sizerc.left, sizerc.bottom, sizerc.right, sizerc.bottom, $$PS_SOLID, 2, 0)
			DrawLine (hDC, sizerc.right, sizerc.top, sizerc.right, sizerc.bottom, $$PS_SOLID, 2, 0)
    END IF

    bErase = $$TRUE

Cleanup:
	SetROP2	(hDC, oldROP)
	ReleaseDC (hCurFrm, hDC)

END FUNCTION

FUNCTION SizeControl ()

	SHARED RECT sizerc
	SHARED hCurCtrl
	SHARED hCurFrm
	
    'Check for the rect being inverted and fix it
    IF sizerc.bottom < sizerc.top THEN
        lTemp = sizerc.top
        sizerc.top = sizerc.bottom
        sizerc.bottom = lTemp
    END IF
    IF sizerc.right < sizerc.left THEN
        lTemp = sizerc.left
        sizerc.left = sizerc.right
        sizerc.right = lTemp
    END IF
    'Make sure we aren't trying to resize the control
    'to 0 width or height
    IF sizerc.right - sizerc.left < 1 THEN
        sizerc.right = sizerc.left + 1
    END IF
    IF sizerc.bottom - sizerc.top < 1 THEN
        sizerc.bottom = sizerc.top + 1
    END IF

     'See the EndSize subroutine
    x = sizerc.left
    y = sizerc.top
    w = sizerc.right - sizerc.left
    h = sizerc.bottom - sizerc.top
		
		MoveWindow (hCurCtrl, x, y, w, h, 1)
		InvalidateRect (hCurCtrl, NULL, 1)
		InvalidateRect (hCurFrm, NULL, 1)
		UpdateWindow (hCurFrm)

END FUNCTION

FUNCTION EndSize ()

	SHARED bSizing
	SHARED hCurCtrl
	SHARED hCurFrm
	SHARED lStartX, lStartY	
	POINT pt
	
	'Halt the sizing operation - called from all drag handles.
	'Clear the resizing rectangle
	DrawSizeRect ($$TRUE)
	'Clear the sizing flag
	bSizing = $$FALSE
	'Resize the control to the size that was requested
	SizeControl ()
	'Reposition the drag handles
	AttachControl (hCurCtrl)
	'Notify parent window that control was modified	
	GetCursorPos (&pt)
	ScreenToClient(hCurFrm, &pt)
	IF pt.x <> lStartX || pt.y <> lStartY THEN
		SendMessageA (hCurFrm, $$SIZER_CONTROLMODIFIED, 0, hCurCtrl)
	END IF
		
END FUNCTION

FUNCTION BeginSize (x, y)

	SHARED bSizing
	SHARED lStartX
	SHARED lStartY
	SHARED RECT sizerc, rc
	
  'Begin the sizing operation - called from all drag handles.
  HideHandles ()
  DrawSizeRect (0)
  bSizing = $$TRUE
  lStartX = x
  lStartY = y
  sizerc = rc

END FUNCTION

FUNCTION BringToFront (hWnd)

	SHARED hCurFrm
	IFZ hWnd THEN RETURN

  SetWindowPos (hWnd, $$HWND_TOP, 0, 0, 0, 0, $$SWP_FRAMECHANGED OR $$SWP_NOSIZE OR $$SWP_NOMOVE)
  InvalidateRect (hWnd, NULL, 1)
  UpdateWindow (hWnd)
	
END FUNCTION


FUNCTION ShowHandles ()

	SHARED handles[]
	
	upp = UBOUND(handles[])
	FOR i = 0 TO upp
		ShowWindow (handles[i], $$SW_SHOWNORMAL)
	NEXT i
	
END FUNCTION
'
' ########################
' #####  MoveHandle  #####
' ########################
'
'
'
FUNCTION MoveHandle (hWnd, x, y)

	RECT handle
	
	GetClientRect (hWnd, &handle)
	
	w = handle.right - handle.left
	h = handle.bottom - handle.top
	MoveWindow (hWnd, x, y, w, h, $$TRUE)
	UpdateWindow (hWnd)
	
END FUNCTION
'
' #######################
' #####  DrawLine  #####
' #######################
'
'
'
FUNCTION DrawLine (hdc, left, top, right, bottom, style, width, color)

  newPen = NULL
  HoldPen = NULL
	
	IFZ width THEN width = 1

  newPen = CreatePen (style, width, color)
	IFZ newPen THEN RETURN ($$TRUE)

  IF newPen THEN
    oldPen = SelectObject (hdc, newPen)
	END IF

  MoveToEx (hdc, left, top, NULL)
  LineTo (hdc, right, bottom)

  IF oldPen THEN 
    SelectObject (hdc, oldPen)
    oldPen = NULL
	END IF

  DeleteObject (newPen)

END FUNCTION
'
' ###########################
' #####  CreateHandles  #####
' ###########################
'
'
'
FUNCTION CreateHandles ()

	SHARED handles[]
	SHARED handleSize, moverSize	
		
	WNDCLASS wc
	SHARED hInst
	SHARED hCurFrm
	SHARED hCurCtrl
	
	DIM handles[8]
	DIM appName$[8]
	DIM appProc[8]
	DIM cursor[8]   
	
	appName$[0] = "TopLeftHandle"
	appProc[0]  = &TopLeftHandleProc()
	cursor[0]   = $$IDC_SIZENWSE
	
	appName$[1] = "TopHandle"
	appProc[1]  = &TopHandleProc()
	cursor[1]   = $$IDC_SIZENS
	
	appName$[2] = "TopRightHandle"
	appProc[2]  = &TopRightHandleProc()
	cursor[2]   = $$IDC_SIZENESW
	
	appName$[3] = "LeftHandle"
	appProc[3]  = &LeftHandleProc()
	cursor[3]   = $$IDC_SIZEWE
	
	appName$[4] = "RightHandle"
	appProc[4]  = &RightHandleProc()
	cursor[4]   = $$IDC_SIZEWE
	
	appName$[5] = "BottomLeftHandle"
	appProc[5]  = &BottomLeftHandleProc()
	cursor[5]   = $$IDC_SIZENESW
	
	appName$[6] = "BottomHandle"
	appProc[6]  = &BottomHandleProc()
	cursor[6]   = $$IDC_SIZENS
	
	appName$[7] = "BottomRightHandle"
	appProc[7]  = &BottomRightHandleProc()
	cursor[7]   = $$IDC_SIZENWSE
	
	appName$[8] = "MoverHandle"
	appProc[8]  = &MoverHandleProc()
	cursor[8]   = $$IDC_SIZEALL
	
	FOR i = 0 TO 8
	
		' Register Handles Window Class
		szAppName$ 				= appName$[i] 
		wc.style 					= $$CS_HREDRAW OR $$CS_VREDRAW
		wc.lpfnWndProc 		= appProc[i] 
		wc.cbClsExtra 		= 0
		wc.cbWndExtra 		= 0
		wc.hInstance 			= hInst
		wc.hIcon 					= NULL
		wc.hCursor 				= LoadCursorA (NULL, cursor[i])
		wc.hbrBackground 	= CreateSolidBrush (RGB (255, 0, 0))
		wc.lpszMenuName 	= NULL
		wc.lpszClassName 	= &szAppName$
		ret = RegisterClassA (&wc)
		
		hsize = handleSize
		IF i = 8 THEN hsize = moverSize

		handles[i] = CreateWindowExA (0, &szAppName$, &"", $$WS_CHILD, _
			0, 0, hsize, hsize, hCurFrm, 9000+i, hInst, NULL)
		BringToFront (handles[i])

	NEXT i

END FUNCTION

FUNCTION TopLeftHandleProc (hWnd, msg, wParam, lParam)

	SHARED bSizing
	SHARED lStartX, lStartY
	SHARED gridSize
	SHARED RECT rc, sizerc
	SHARED hCurCtrl
	RECT rect
	SHARED hCurFrm
	POINT pt

	SELECT CASE msg
'		CASE $$WM_PAINT				: 

		CASE $$WM_LBUTTONDOWN : 
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			SetCapture (hWnd)
			BeginSize (pt.x, pt.y)

		CASE $$WM_LBUTTONUP 	: 
			EndSize ()
			ReleaseCapture ()
			
		CASE $$WM_MOUSEMOVE 	: 
			IF bSizing = $$FALSE THEN RETURN
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			DrawSizeRect ($$TRUE)
			gx = gridSize
			sizerc.left = ((rc.left - (lStartX - pt.x)) \ gx) * gx
			sizerc.top = ((rc.top - (lStartY - pt.y)) \ gx) * gx
			DrawSizeRect (0)
			
		CASE ELSE
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)
	END SELECT

END FUNCTION

FUNCTION LeftHandleProc (hWnd, msg, wParam, lParam)

	SHARED bSizing
	SHARED lStartX, lStartY
	SHARED gridSize
	SHARED RECT rc, sizerc
	SHARED hCurCtrl
	RECT rect
	SHARED hCurFrm
	POINT pt

	SELECT CASE msg
'		CASE $$WM_PAINT				: 

		CASE $$WM_LBUTTONDOWN : 
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			SetCapture (hWnd)
			BeginSize (pt.x, pt.y)

		CASE $$WM_LBUTTONUP 	: 
			EndSize ()
			ReleaseCapture ()
			
		CASE $$WM_MOUSEMOVE 	: 
			IF bSizing = $$FALSE THEN RETURN
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			DrawSizeRect ($$TRUE)
			gx = gridSize
			sizerc.left = ((rc.left - (lStartX - pt.x)) \ gx) * gx
			DrawSizeRect (0)
			
		CASE ELSE
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)
	END SELECT

END FUNCTION

FUNCTION TopRightHandleProc (hWnd, msg, wParam, lParam)

	SHARED bSizing
	SHARED lStartX, lStartY
	SHARED gridSize
	SHARED RECT rc, sizerc
	SHARED hCurCtrl
	RECT rect
	SHARED hCurFrm
	POINT pt

	SELECT CASE msg
'		CASE $$WM_PAINT				: 

		CASE $$WM_LBUTTONDOWN : 
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			SetCapture (hWnd)
			BeginSize (pt.x, pt.y)

		CASE $$WM_LBUTTONUP 	: 
			EndSize ()
			ReleaseCapture ()
			
		CASE $$WM_MOUSEMOVE 	: 
			IF bSizing = $$FALSE THEN RETURN
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			DrawSizeRect ($$TRUE)
			gx = gridSize
			sizerc.right = ((rc.right - (lStartX - pt.x)) \ gx) * gx
			sizerc.top = ((rc.top - (lStartY - pt.y)) \ gx) * gx
			DrawSizeRect (0)
			
		CASE ELSE
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)
	END SELECT

END FUNCTION

FUNCTION TopHandleProc (hWnd, msg, wParam, lParam)

	SHARED bSizing
	SHARED lStartX, lStartY
	SHARED gridSize
	SHARED RECT rc, sizerc
	SHARED hCurCtrl
	RECT rect
	SHARED hCurFrm
	POINT pt

	SELECT CASE msg
'		CASE $$WM_PAINT				: 

		CASE $$WM_LBUTTONDOWN : 
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			SetCapture (hWnd)
			BeginSize (pt.x, pt.y)

		CASE $$WM_LBUTTONUP 	: 
			EndSize ()
			ReleaseCapture ()
			
		CASE $$WM_MOUSEMOVE 	: 
			IF bSizing = $$FALSE THEN RETURN
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			DrawSizeRect ($$TRUE)
			gy = gridSize
			sizerc.top = ((rc.top - (lStartY - pt.y)) \ gy) * gy
			DrawSizeRect (0)
			
		CASE ELSE
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)
	END SELECT

END FUNCTION

FUNCTION RightHandleProc (hWnd, msg, wParam, lParam)

	SHARED bSizing
	SHARED lStartX, lStartY
	SHARED gridSize
	SHARED RECT rc, sizerc
	SHARED hCurCtrl
	RECT rect
	SHARED hCurFrm
	POINT pt

	SELECT CASE msg
'		CASE $$WM_PAINT				: 

		CASE $$WM_LBUTTONDOWN : 
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			SetCapture (hWnd)
			BeginSize (pt.x, pt.y)

		CASE $$WM_LBUTTONUP 	: 
			EndSize ()
			ReleaseCapture ()
			
		CASE $$WM_MOUSEMOVE 	: 
			IF bSizing = $$FALSE THEN RETURN
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			DrawSizeRect ($$TRUE)
			gx = gridSize
			sizerc.right = ((rc.right - (lStartX - pt.x)) \ gx) * gx
			DrawSizeRect (0)
			
		CASE ELSE
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)
	END SELECT

END FUNCTION

FUNCTION BottomLeftHandleProc (hWnd, msg, wParam, lParam)

	SHARED bSizing
	SHARED lStartX, lStartY
	SHARED gridSize
	SHARED RECT rc, sizerc
	SHARED hCurCtrl
	RECT rect
	SHARED hCurFrm
	POINT pt

	SELECT CASE msg
'		CASE $$WM_PAINT				: 

		CASE $$WM_LBUTTONDOWN : 
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			SetCapture (hWnd)
			BeginSize (pt.x, pt.y)

		CASE $$WM_LBUTTONUP 	: 
			EndSize ()
			ReleaseCapture ()
			
		CASE $$WM_MOUSEMOVE 	: 
			IF bSizing = $$FALSE THEN RETURN
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			DrawSizeRect ($$TRUE)
			gx = gridSize
			sizerc.left = ((rc.left - (lStartX - pt.x)) \ gx) * gx
			sizerc.bottom = ((rc.bottom - (lStartY - pt.y)) \ gx) * gx
			DrawSizeRect (0)
			
		CASE ELSE
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)
	END SELECT

END FUNCTION

FUNCTION BottomHandleProc (hWnd, msg, wParam, lParam)

	SHARED bSizing
	SHARED lStartX, lStartY
	SHARED gridSize
	SHARED RECT rc, sizerc
	SHARED hCurCtrl
	RECT rect
	SHARED hCurFrm
	POINT pt

	SELECT CASE msg
'		CASE $$WM_PAINT				: 

		CASE $$WM_LBUTTONDOWN : 
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			SetCapture (hWnd)
			BeginSize (pt.x, pt.y)

		CASE $$WM_LBUTTONUP 	: 
			EndSize ()
			ReleaseCapture ()
			
		CASE $$WM_MOUSEMOVE 	: 
			IF bSizing = $$FALSE THEN RETURN
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			DrawSizeRect ($$TRUE)
			gy = gridSize
			sizerc.bottom = ((rc.bottom - (lStartY - pt.y)) \ gy) * gy
			DrawSizeRect (0)
			
		CASE ELSE
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)
	END SELECT

END FUNCTION

FUNCTION BottomRightHandleProc (hWnd, msg, wParam, lParam)

	SHARED bSizing
	SHARED lStartX, lStartY
	SHARED gridSize
	SHARED RECT rc, sizerc
	SHARED hCurCtrl
	RECT rect
	SHARED hCurFrm
	POINT pt

	SELECT CASE msg
'		CASE $$WM_PAINT				: 

		CASE $$WM_LBUTTONDOWN : 
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			SetCapture (hWnd)
			BeginSize (pt.x, pt.y)

		CASE $$WM_LBUTTONUP 	: 
			EndSize ()
			ReleaseCapture ()
			
		CASE $$WM_MOUSEMOVE 	: 
			IF bSizing = $$FALSE THEN RETURN
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			DrawSizeRect ($$TRUE)
			gx = gridSize
			sizerc.right = ((rc.right - (lStartX - pt.x)) \ gx) * gx
			sizerc.bottom = ((rc.bottom - (lStartY - pt.y)) \ gx) * gx
			DrawSizeRect (0)
			
		CASE ELSE
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)
	END SELECT

END FUNCTION

FUNCTION MoverHandleProc (hWnd, msg, wParam, lParam)

	SHARED bSizing
	SHARED lStartX, lStartY
	SHARED gridSize
	SHARED RECT rc, sizerc
	SHARED hCurCtrl
	RECT rect
	SHARED hCurFrm
	POINT pt

	SELECT CASE msg
'		CASE $$WM_PAINT				: 

		CASE $$WM_LBUTTONDOWN : 
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			SetCapture (hWnd)
			BeginSize (pt.x, pt.y)

		CASE $$WM_LBUTTONUP 	: 
			EndSize ()
			ReleaseCapture ()
			
		CASE $$WM_MOUSEMOVE 	: 
			IF bSizing = $$FALSE THEN RETURN
			pt.x = LOWORD (lParam)
			pt.y = HIWORD (lParam)
			MapWindowPoints (hWnd, hCurFrm, &pt, 1) 
			DrawSizeRect ($$TRUE)
			gx = gridSize
			sizerc.left = ((rc.left - (lStartX - pt.x)) \ gx) * gx
			sizerc.top = ((rc.top - (lStartY - pt.y)) \ gx) * gx
			GetClientRect (hCurCtrl, &rect)
			sizerc.right = sizerc.left + (rect.right - rect.left)
			sizerc.bottom = sizerc.top + (rect.bottom - rect.top)
			DrawSizeRect (0)
	
		CASE ELSE
			RETURN DefWindowProcA (hWnd, msg, wParam, lParam)
	END SELECT

END FUNCTION
'
' ############################
' #####  DestroyHandles  #####
' ############################
'
'
'
FUNCTION DestroyHandles ()

	SHARED handles[]
	
	FOR i = 0 TO 8
		DestroyWindow (handles[i])
	NEXT i
	
	DIM handles[]

END FUNCTION
'
' ###########################
' #####  SetHandleSize  #####
' ###########################
'
'
'
FUNCTION SetHandleSize (size)

	SHARED handleSize
	IF size <=0 THEN RETURN
	handleSize = size

END FUNCTION
'
' #########################
' #####  SetGridSize  #####
' #########################
'
'
'
FUNCTION SetGridSize (size)

	SHARED gridSize
	
	IF size <= 0 THEN RETURN
	gridSize = size

END FUNCTION
'
' #############################
' #####  SetHandlehandleOffset  #####
' #############################
'
'
'
FUNCTION SetHandleOffset (offset)

	SHARED handleOffset
	handleOffset = offset

END FUNCTION



END PROGRAM